/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  F. Blomquist, University of Wuppertal, 20.09.2007;
*/

/* CVS $Id: lx_real.inl,v 1.3 2006/12/06 18:10:25 cxsc Exp $ */

namespace cxsc {

inline int StagPrec(const lx_real &a) throw() 
{ return StagPrec(a.lr); }

inline int expo(const lx_real &a) throw()
{ return (a.ex); }

inline int sign(const lx_real &a) throw() 
{ return sign(a.lr); }

inline l_real lr_part(const lx_real &a) throw()
{ return (a.lr); }

inline lx_real abs(const lx_real& a) throw()
{ return lx_real(a.ex,abs(a.lr)); }

inline lx_real & lx_real::operator = (const lx_real &a) throw()
{
    ex = a.ex;
    lr = a.lr;
    return *this;
}

inline lx_real & lx_real::operator = (const l_real &a) throw()
{
    ex = 0;
    lr = a;
    return *this;
}

inline lx_real & lx_real::operator = (const real &a) throw()
{
    ex = 0;
    lr = a;
    return *this;
}

// -----------------------------------------------------

inline bool eq_zero(const lx_real &a) throw()
{  return (a.lr == 0 );  }

inline bool gr_zero(const lx_real &a) throw()
{  return (a.lr > 0 );  }

inline bool ge_zero(const lx_real &a) throw()
{  return (a.lr >= 0);  }

inline bool sm_zero(const lx_real &a) throw()
{  return (a.lr < 0 );  }

inline bool se_zero(const lx_real &a) throw()
{  return (a.lr <= 0);  }

// -----------------------------------------------------

inline lx_real operator -(const lx_real &a) throw()
{ return lx_real(a.ex,-a.lr); }

inline lx_real operator +(const lx_real &a) throw()
{ return a; }

inline bool operator == (const lx_real &a, const l_real &b) throw()
{  return (a==lx_real(b));  }

inline bool operator == (const l_real &a, const lx_real &b) throw()
{  return (lx_real(a)==b);  }

inline bool operator == (const lx_real &a, const real &b) throw()
{  return (a==lx_real(b));  }

inline bool operator == (const real &a, const lx_real &b) throw()
{  return (lx_real(a)==b);  }


inline bool operator != (const lx_real &a, const lx_real &b) throw()
{  return !(a==b); }

inline bool operator != (const lx_real &a, const l_real &b) throw()
{  return !(a==lx_real(b));  }

inline bool operator != (const l_real &a, const lx_real &b) throw()
{  return !(lx_real(a)==b);  }

inline bool operator != (const lx_real &a, const real &b) throw()
{  return !(a==lx_real(b));  }

inline bool operator != (const real &a, const lx_real &b) throw()
{  return !(lx_real(a)==b);  }

inline bool operator <= (const lx_real &a, const lx_real &b) throw()
{ return !(a>b); }

inline bool operator < (const lx_real &a, const lx_real &b) throw()
{  return (b>a); }

inline bool operator >= (const lx_real &a, const lx_real &b) throw()
{ return !(a<b); }

// ---------------------------------------------------------

inline bool operator > (const real &a, const lx_real &b) throw()
{ return lx_real(a)>b; }

inline bool operator <= (const real &a, const lx_real &b) throw()
{ return !(a>b); }

inline bool operator <  (const real &a, const lx_real &b) throw()
{  return b>lx_real(a); }

inline bool operator >= (const real &a, const lx_real &b) throw()
{ return !(a<b); }

// ---------------------------------------------------------

inline bool operator >  (const lx_real &a, const real &b) throw()
{ return a>lx_real(b); }

inline bool operator <= (const lx_real &a, const real &b) throw()
{ return !(a>b); }

inline bool operator <  (const lx_real &a, const real &b) throw()
{  return b>a; }

inline bool operator >= (const lx_real &a, const real &b) throw()
{ return !(a<b); }

// ---------------------------------------------------------

inline bool operator >  (const l_real &a, const lx_real &b) throw()
{ return lx_real(a)>b; }

inline bool operator <= (const l_real &a, const lx_real &b) throw()
{ return !(a>b); }

inline bool operator <  (const l_real &a, const lx_real &b) throw()
{  return b>lx_real(a); }

inline bool operator >= (const l_real &a, const lx_real &b) throw()
{ return !(a<b); }

// ---------------------------------------------------------

inline bool operator >  (const lx_real &a, const l_real &b) throw()
{ return a>lx_real(b); }

inline bool operator <= (const lx_real &a, const l_real &b) throw()
{ return !(a>b); }

inline bool operator <  (const lx_real &a, const l_real &b) throw()
{  return b>a; }

inline bool operator >= (const lx_real &a, const l_real &b) throw()
{ return !(a<b); }


// -----------------------------------------------------

inline std::ostream& operator << (std::ostream& s, const lx_real& a) 
    throw()
// A value a of type lx_real is written to the output channel.
// The output has the form:  {(ex),lr}
{     
    s << '{' 
      << '('
      << a.ex 
      << ")" 
      << ", " 
      << a.lr       
      << '}';
    return s;
}

inline std::string & operator << (std::string &s, const lx_real& a) 
    throw()
// The value of a variable a of type lx_real is copied to a string s.
// s has the form:  {(ex),lr}
{  
    std::stringstream ss;
    string str;

    str = "{(";
    s += str;
    ss << a.ex;
    ss >> str;
    s += str;
    s+="), "; 
    s << a.lr; 
    s+='}';
    return s;
}

inline int add_int(int a,int b)
// Calculating a+b; abortion by int overflow.
{
   int res(a+b);
   bool bl1(a>0 && b>0 && res<a),
        bl2(a<0 && b<0 && res>a);
   if (bl1||bl2)
   {
       std::cerr << "ERROR in int add_int(int,int): Overflow!" << std::endl;
       exit(1);	 
   }
   return res;
}

inline int sub_int(int a,int b)
// Calculating a-b; abortion by int overflow.
{
   if (b<0 && -b<0)
   { 
       std::cerr << "ERROR in int sub_int(int,int): Overflow!" << std::endl;
       exit(1);
   }
   return add_int(a,-b);
}

inline lx_real max(const lx_real& a, const lx_real& b)
    { return (b>a)? b : a; }

inline lx_real min(const lx_real& a, const lx_real& b)
    { return (b>a)? a : b; }

} // end namespace cxsc
